@@ -520,13 +520,18 @@ def message_excepthook(type, value, tb):
     """
     try:
         import traceback
+        import qtalchemy.xplatform as xp
 
         msgBox = QtGui.QMessageBox()
         msgBox.setWindowTitle(QtGui.QApplication.applicationName())
         msgBox.setIcon(QtGui.QMessageBox.Critical)
         
-        tb2 = "An unhandled error was encountered.  Consider copying this entire message to an email to the program author by pressing Ctrl+C on this message.  Include a brief description of the actions leading to this error.\n\n{0}" \
-                    .format(''.join(traceback.format_exception(type, value, tb, limit=None)))
+        if xp.is_windows():
+            body = "An unhandled error was encountered.  Consider copying this entire message to an email to the program author by pressing Ctrl+C on this message.  Include a brief description of the actions leading to this error.\n\n{0}"
+        else:
+            body = "An unhandled error was encountered. Consider copying this entire message to an email to the program author.  Include a brief description of the actions leading to this error.\n\n{0}"
+
+        tb2 = body.format(''.join(traceback.format_exception(type, value, tb, limit=3)))
         
         msgBox.setText( tb2 )
         
