@@ -2,7 +2,7 @@
 ##############################################################################
 #       Copyright (C) 2010, Joel B. Mohler <joel@kiwistrawberry.us>
 #
-#  Distributed under the terms of the GNU General Public License (GPL)
+#  Distributed under the terms of the GNU Lesser General Public License (LGPL)
 #                  http://www.gnu.org/licenses/
 ##############################################################################
 """
@@ -30,36 +30,143 @@ else:
     sqlalchemy_to_python_type = [(satype.__class__,pytype) for pytype,satype in sqlalchemy.types._type_map.iteritems()]
 sqlalchemy_to_python_type.append((sqlalchemy.types.String,str))
 
-def BoundPropertyAttribute(classMember, attribute):
+def SA2PY(satype):
+    for sa,py in sqlalchemy_to_python_type:
+        if isinstance(satype,sa):
+            return py
+    return None
+
+def BoundPropertyAttribute(classMember, attribute, SA2R=None, PY2R=None):
     """
     Abstracts out property determination code between a UserAttr and SA column.
 
-    Attributes like the label, yoke and what's this string are kept in the 
-    SA info dictionary for a column.  They are kept directly as members of the 
+    Attributes like the label, yoke and what's this string are kept in the SA
+    info dictionary for a column.  They are kept directly as members of the
     descriptor class for a UserAttr.
+    
+    Some values are easily defaulted intelligently from the SQLAlchemy types or
+    the basic python types of the column.  The callables SA2R, SA2PY and PY2R
+    extend this function with the ability to do so.
+
+    :param classMember:  UserAttr, ModelColumn or SQLalchemy column
+        attribute to introspect
+    :param attribute:  The attribute value to compute
+    :param SA2R:  a callable taking the SQLAlchemy column type and returning
+        the attribute value (may be None or return None) 
+    :param PY2R:  a callable converting a python column type and returning the
+        attribute value (may be None or return None)
     """
+    type_=None
+    result = None
+
+    # unspecified UserAttr class
+    if hasattr(classMember, attribute):
+        result = getattr(classMember, attribute)
+    if result is None and PY2R is not None:
+        try:
+            type_ = classMember.base_type
+        except AttributeError as e:
+            pass
 
     # classMember appears like a SA column
     if hasattr(classMember,"label") and callable(classMember.label):
-        col=classMember.property.columns[0]
+        col = classMember.property.columns[0]
         if hasattr(col,"info") and col.info.has_key(attribute):
             return col.info[attribute]
+        else:
+            if SA2R is not None:
+                result = SA2R(classMember.property.columns[0].type)
+            if result is not None:
+                return result
+            if PY2R is not None:
+                type_ = SA2PY(classMember.property.columns[0].type)
 
+    if type_ is not None and PY2R is not None:
+        return PY2R(type_)
+
+    return result
+
+def attr(cls, col, attribute, CAcallable):
     try:
-        # cls has an attribute col that looks like a UserAttr
-        return getattr(classMember, attribute)
+        # col looks like a ModelColumn
+        return getattr(col, attribute)
     except AttributeError:
-        pass
+        assert cls is not None, "If 'col' is not (like) a ModelColumn, then cls must be specified."
+
+    return CAcallable(getattr(cls,col))
+
+def ClassAttributeEmptyZero(classMember):
+    return BoundPropertyAttribute(classMember, "empty_zero")
+
+attrEmptyZero = lambda cls, col: attr(cls, col, "empty_zero", ClassAttributeEmptyZero)
 
-    return None
 
 def ClassAttributeWhatsThis(classMember):
     return BoundPropertyAttribute(classMember, "whats_this")
 
+attrWhatsThis = lambda cls, col: attr(cls, col, "whats_this", ClassAttributeWhatsThis)
+
+def ClassAttributeAlignment(classMember):
+    """
+    :param classMember: The attribute in the class (not the instance)
+    
+    This function
+    
+    >>> class X:
+    ...     name=UserAttr(str,"First Name")
+    ...     age=UserAttr(AttrNumeric(3),"Precise Age")
+    >>> ClassAttributeAlignment(X.name)
+    'left'
+    >>> ClassAttributeAlignment(X.age)
+    'right'
+
+    >>> from sqlalchemy import Table, Column, Integer, String, Text, Date, Time, Boolean, Numeric, MetaData
+    >>> from sqlalchemy.ext.declarative import declarative_base
+    >>> metadata = MetaData()
+    >>> Base = declarative_base(metadata=metadata)
+    >>> class Books(Base):
+    ...     __table__ = Table('books', metadata,
+    ...             Column('id', Integer, primary_key=True),
+    ...             Column('author', String(50)),
+    ...             Column('release', Date()),
+    ...             Column('release_time', Time()),
+    ...             Column('copies', Integer),
+    ...             Column('juvenile', Boolean),
+    ...             Column('review', Text),
+    ...             Column('price', Numeric(precision=10,scale=2,asdecimal=True)),
+    ...             Column('title', String(50), info={'yoke': 'text'}))
+    >>> ClassAttributeAlignment(Books.author)
+    'left'
+    >>> ClassAttributeAlignment(Books.release)
+    'left'
+    >>> ClassAttributeAlignment(Books.release_time)
+    'left'
+    >>> ClassAttributeAlignment(Books.copies)
+    'right'
+    >>> ClassAttributeAlignment(Books.price)
+    'right'
+    >>> ClassAttributeAlignment(Books.title)
+    'left'
+    >>> ClassAttributeAlignment(Books.review)
+    'left'
+    >>> ClassAttributeAlignment(Books.juvenile)
+    'left'
+    """
+    def PY2R(type_):
+        if type_ in (decimal.Decimal,float,int) or isinstance(type_, AttrNumeric):
+            return "right"
+        else:
+            return "left"
+
+    return BoundPropertyAttribute(classMember, "alignment", PY2R=PY2R)
+
+attrAlignment = lambda cls, col: attr(cls, col, "alignment", ClassAttributeAlignment)
+
 def UserLabel(identifier):
     """
-    Convert a python or SQL identifier name to a user viewable label.  Essentially this makes word breaks 
-    at '_' characters and uses initial caps for words.
+    Convert a python or SQL identifier name to a user viewable label.
+    Essentially this makes word breaks at '_' characters and uses initial caps
+    for words.
     
     >>> UserLabel("balance_sheet")
     'Balance Sheet'
@@ -182,7 +289,9 @@ def ClassAttributeYoke(classMember):
     try:
         # unspecified UserAttr class
         type_ = classMember.base_type
-        return classMember.yoke_specifier()
+        y = classMember.yoke_specifier()
+        if y is not None:
+            return y
     except AttributeError, e:
         pass
 
@@ -415,12 +524,15 @@ def AttrWidgetFactory(userattr,parent=None, index=None):
 
 
 class ModelColumn:
-    def __init__(self, attr, type_, label=None, readonly=False, width=None):
+    def __init__(self, attr, type_, label=None, readonly=False, width=None, empty_zero=True, alignment=None):
         """
         :param attr:  attribute name
         :param label:  label to display in the list header
         :param readonly:  true if the column should not be editable even if the view is
         :param width:  width in pixels (?) or relative width to other columns
+        :param empty_zero:  display 0's as empty strings in display mode if true 
+        :param alignment:  preferred alignment of text in the column
+            options include "right", "left", "top-left" and the general form is "vertical-horizontal" with the vertical part being optional
         """
         self.attr = attr
         if label is None:
@@ -429,7 +541,11 @@ class ModelColumn:
             self.label = label
         self.type_ = type_
         self.readonly = readonly
+        self.empty_zero = empty_zero
         self.width = width
+        if type_ in (int, float, decimal.Decimal) and alignment is None:
+            alignment = "right"
+        self.alignment = alignment
 
 
 class ObjectRepresent(object):
@@ -540,7 +656,7 @@ def ClassTableModel(cls, attributes, readonly=True, fixed_rows=False, row_manage
         >>> from sqlalchemy.ext.declarative import declarative_base
         >>> 
         >>> from qtalchemy import QueryTableModel, PBSessionMaker, ModelObject
-        >>> from PyQt4 import QtCore, QtGui
+        >>> from PySide import QtCore, QtGui
         >>> 
         >>> metadata = MetaData()
         >>> Base = declarative_base(metadata=metadata,cls=ModelObject)
@@ -591,7 +707,7 @@ def ClassTableModel(cls, attributes, readonly=True, fixed_rows=False, row_manage
     """
     def makeModelColumn(cls, attr):
         ua = getattr(cls, attr)
-        return ModelColumn(attr, ClassAttributeType(ua), ClassAttributeLabel(ua), readonly=attrReadonly(cls, attr))
+        return ModelColumn(attr, ClassAttributeType(ua), ClassAttributeLabel(ua), readonly=attrReadonly(cls, attr), alignment=attrAlignment(cls, attr))
     cols = [makeModelColumn(cls, a) for a in attributes]
     return PBTableModel(cols, cls=cls, readonly=readonly, fixed_rows=fixed_rows, row_manager=row_manager, objectConverter=objectConverter, tree_structure=tree_structure)
 
@@ -637,7 +753,10 @@ def QueryTableModel(q, cols=None, include_uuid=False, ssrc=None, objectConverter
             match = False
             for satype,pytype in sqlalchemy_to_python_type:
                 if isinstance(col.type,satype):
-                    model_cols.append(ModelColumn(col.name,pytype))
+                    alignment = "left"
+                    if pytype in (int, float, decimal.Decimal):
+                        alignment="right"
+                    model_cols.append(ModelColumn(col.name,pytype,alignment=alignment))
                     match=True
                     break
             assert match, "The sqlalchemy type %s was not known for mapping to a python type." % col.type
@@ -869,17 +988,28 @@ class PBTableModel(QtCore.QAbstractItemModel):
 
         if role == QtCore.Qt.DisplayRole:
             if primary_yoke != 'bool':
-                return toQType(primary_value,suggested=primary_type)
+                return toQType(primary_value, suggested=primary_type, empty_zero=attrEmptyZero(self.cls, c))
         elif role == QtCore.Qt.EditRole:
             return toQType(primary_value,suggested=primary_type)
         elif role == QtCore.Qt.CheckStateRole:
             if primary_yoke == 'bool':
                 return QtCore.Qt.Checked if primary_value else QtCore.Qt.Unchecked
         elif role == QtCore.Qt.TextAlignmentRole:
+            align = attrAlignment(self.cls, c)
+            hori = "left"
+            vert = "vcenter"
+            if align is not None:
+                t = align.split('-', 1)
+                if len(t) == 1:
+                    hori = align
+                else:
+                    hori, vert = t
+            # should not have to cast to int in either of these returns
+            # see https://bugreports.qt-project.org/browse/PYSIDE-20
             if hasattr(r,"textAlignmentRole"):
-                return r.textAlignmentRole(c)
-            if primary_yoke in ('int','float'):
-                return QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter
+                return int(r.textAlignmentRole(c))
+            return int({"left": QtCore.Qt.AlignLeft, "hcenter": QtCore.Qt.AlignHCenter, "right": QtCore.Qt.AlignRight}[hori] | \
+                     {"top": QtCore.Qt.AlignTop, "vcenter": QtCore.Qt.AlignVCenter, "bottom": QtCore.Qt.AlignBottom}[vert])
         elif role == QtCore.Qt.FontRole:
             if hasattr(r,"fontRole"):
                 return r.fontRole(c)
@@ -991,7 +1121,10 @@ def modelMimeRectangle(model, selected):
         for x in full_cols:
             r = ""
             if x in my_cols:
-                r = str(fromQType(model.data(my_cols[x], QtCore.Qt.DisplayRole)))
+                cellData = fromQType(model.data(my_cols[x], QtCore.Qt.DisplayRole))
+                if cellData is None and model.data(my_cols[x], QtCore.Qt.CheckStateRole) is not None:
+                    cellData = model.data(my_cols[x], QtCore.Qt.CheckStateRole) == QtCore.Qt.Checked
+                r = str(cellData)
             result.append(r)
         return result
 
@@ -1082,11 +1215,15 @@ class WidgetAttributeMapper(object):
                 return y[0].mainWidget()
         return None
 
+    def connected(self):
+        return self.obj is not None
+
     def getObjectAttr(self,attr):
         return getattr(self.obj,attr)
 
     def setObjectAttr(self,attr,value):
-        setattr(self.obj,attr,value)
+        if not hasattr(self.obj, "is_setting") or not self.obj.is_setting(attr):
+            setattr(self.obj,attr,value)
 
     def row_object_change(self, obj, attr, oldvalue):
         assert obj is self.obj
@@ -1180,8 +1317,9 @@ class WidgetAttributeMapper(object):
         return colIndex
 
     def submit(self):
-        for y in self.yokes:
-            y.Save()
+        if self.obj is not None:
+            for y in self.yokes:
+                y.Save()
 
     def bind(self, widget, attr, yoke_factory=None):
         import input_yoke
