@@ -228,7 +228,7 @@ class DomainEntity(CommandMenu):
     commands associated with any of the tables in the query.
     """
 
-    def __init__(self, info={}):
+    def __init__(self, info=None):
         """
         The following assignments are a common proto-type associating this DomainEntity with a 
         explicit table.  We do not yet define the sql joins clearly.
@@ -237,7 +237,7 @@ class DomainEntity(CommandMenu):
         self.key_column = None
         self.list_display_columns = None
         self.list_search_columns = None
-        self.info = info
+        self.info = {} if info is None else info
 
     def list_query_converter(self):
         from sqlalchemy.orm import Query
