@@ -8,7 +8,7 @@ from objectforms import MapperDialog, BoundDialog, BoundDialogObject, \
                         messaged_commit, exception_message_box, appMessage, SessionMessageBox
 
 try:
-    from PyQt4 import QtDeclarative
+    from PySide import QtDeclarative
     from quicklist import QuickList
 except ImportError, e:
     pass
