@@ -2,14 +2,14 @@
 ##############################################################################
 #       Copyright (C) 2010, Joel B. Mohler <joel@kiwistrawberry.us>
 #
-#  Distributed under the terms of the GNU General Public License (GPL)
+#  Distributed under the terms of the GNU Lesser General Public License (LGPL)
 #                  http://www.gnu.org/licenses/
 ##############################################################################
 
 from qtalchemy import *
 from qtalchemy.widgets import TableView
 from qtalchemy.dialogs import *
-from PyQt4 import QtCore, QtGui
+from PySide import QtCore, QtGui
 from sqlalchemy.orm.attributes import InstrumentedAttribute
 import os.path
 import decimal
@@ -175,7 +175,7 @@ class ImportWizard(QtGui.QWizard):
     >>> 
     >>> from qtalchemy import *
     >>> from qtalchemy.ext.dataimport import *
-    >>> from PyQt4 import QtCore, QtGui
+    >>> from PySide import QtCore, QtGui
     >>> 
     >>> metadata = MetaData()
     >>> Base = declarative_base(metadata=metadata, cls=ModelObject)
