@@ -2,7 +2,7 @@
 ##############################################################################
 #       Copyright (C) 2010, Joel B. Mohler <joel@kiwistrawberry.us>
 #
-#  Distributed under the terms of the GNU General Public License (GPL)
+#  Distributed under the terms of the GNU Lesser General Public License (LGPL)
 #                  http://www.gnu.org/licenses/
 ##############################################################################
 
@@ -30,7 +30,7 @@ class ViewBase(object):
         elif index.row() > 0:
             return model.index(index.row()-1,model.columnCount(index.parent())-1,index.parent())
         else:
-            return model.index(index.rowCount(index.parent())-1,model.columnCount(index.parent())-1,index.parent())
+            return model.index(model.rowCount(index.parent())-1,model.columnCount(index.parent())-1,index.parent())
 
     def delKeyPressed(self):
         #print "deleting"
